from models import Task, TaskCollection
from utils import ignored, humanize_duration
import click
import os


class Data(object):
    def __init__(self, file_path=None):
        self.trask_file = os.path.expanduser(file_path)
        self.tasks = TaskCollection.from_file(self.trask_file)


CONTEXT_SETTINGS = {
    "help_option_names": ["-h", "--help"],
}


@click.group(context_settings=CONTEXT_SETTINGS)
@click.option('file_path', '-f', '--file',
              type=click.Path(),
              required=False,
              default='~/.trasks.json',
              envvar='TRASK_HOME',
              help='File to be used for storing tasks. '
              'Default is ~/.trasks.json')
@click.pass_context
def main(ctx, file_path):
    """Simple command line tool to track your tasks."""
    ctx.obj = Data(file_path=file_path)


@main.command()
@click.argument('task', metavar='<task>')
@click.pass_obj
def start(data, task):
    """Start a new task or resume a stopped one.

    The argument <task> is either a task ID or task name.
    """
    for t in data.tasks.itervalues():
        if task not in [t.name, t.tid]:
            with ignored(Exception):
                t.stop()
    try:
        data.tasks[task].start()
    except KeyError:
        new_task = Task(task)
        new_task.start()
        data.tasks[new_task.tid] = new_task
    except Exception as e:
        click.echo(e.message)
    data.tasks.to_file(data.trask_file)


@main.command()
@click.argument('task', metavar='<task>')
@click.pass_obj
def stop(data, task):
    """Stop a running task.

    The argument <task> is either a task ID or task name.
    """
    try:
        data.tasks[task].stop()
    except KeyError as e:
        click.echo('Task "{}" does not exist!'.format(task))
    except Exception as e:
        click.echo(e.message)
    data.tasks.to_file(data.trask_file)


@main.command()
@click.pass_obj
def status(data):
    """List all tasks."""
    thead = '{:10}   {:40.40}   {:20}   {:20}   {:20}   {:15}   {:10}'
    click.echo(thead.format('ID', 'NAME', 'CREATED_AT', 'STARTED_AT',
                            'STOPPED_AT', 'DURATION', 'IS_STARTED'))
    trow = ('{id:10}   {name:40.40}   {created_at:20}   {started_at:20}   '
            '{stopped_at:20}   {duration:15}   {is_started:>10}')
    for task in data.tasks.itervalues():
        values = task.to_dict()
        values['name'] = '"{}"'.format(values['name'])
        values['duration'] = humanize_duration(values['duration'])
        values['is_started'] = 'YES' if values['is_started'] else 'NO'
        if not values['stopped_at']:
            values['stopped_at'] = 'NEVER'
        click.echo(trow.format(**values))
    data.tasks.to_file(data.trask_file)


if __name__ == '__main__':
    main()
