from collections import MutableMapping
from datetime import datetime
from utils import generate_id, date_from_string, date_to_string
import json

BIG_BANG = datetime(1, 1, 1)


class Task(object):
    """Represent a task object."""

    def __init__(self, name):
        self.name = name
        self.tid = generate_id(self.name)
        self.is_started = False
        self.created_at = datetime.utcnow()
        self.started_at = None
        self.stopped_at = None
        self.last_updated = None
        self.duration = 0

    @classmethod
    def from_dict(cls, task_dict):
        """Create a Task object from a dictionary.

        :param task_dict: The dictionary to create the object from.
        :type task_dict: dict
        :returns: A new task
        :rtype: :class:`~.Task`

        :Example:

        >>> task = Task.from_dict(
            {'name': 'new task',
             'is_started': False,
             'created_at': '2015-05-17T20:00:00',
             'started_at': '2015-05-17T20:00:00',
             'stopped_at': '2015-05-17T21:00:00',
             'duration': 3600}
        )
        >>> type(task)
        <class 'trask.models.Task'>

        .. note:: If not provided, `is_started` defaults to False,
            timestamps default to None and `duration` defaults to 0.
        """
        t = Task(task_dict['name'])
        t.is_started = task_dict.get('is_started', False)
        t.created_at = date_from_string(task_dict.get('created_at'))
        t.started_at = date_from_string(task_dict.get('started_at'))
        t.stopped_at = date_from_string(task_dict.get('stopped_at'))
        t.last_updated = date_from_string(task_dict.get('last_updated'))
        t.duration = task_dict.get('duration', 0)
        return t

    def to_dict(self):
        """Update the duration and get the task as a dictionary.

        :returns: The task in a dictionary format.
        :rtype: dict
        """
        self._update_duration()
        return {
            'id': self.tid,
            'name': self.name,
            'is_started': self.is_started,
            'created_at': date_to_string(self.created_at),
            'started_at': date_to_string(self.started_at),
            'stopped_at': date_to_string(self.stopped_at),
            'last_updated': date_to_string(self.last_updated),
            'duration': self.duration,
        }

    def start(self, at=None):
        """Start the task.

        :param at: Exact timestamp of the start time, defaults to now.
        :type at: `datetime.datetime`
        :raises Exception: If the task is already started.
        """
        if self.is_started:
            raise Exception("Task with ID %s already started." % self.tid)

        self.started_at = self.last_updated = at or datetime.utcnow()
        self.is_started = True

    def stop(self, at=None):
        """Stop the task.

        :param at: Exact timestamp of the stop time, defaults to now.
        :type at: `datetime.datetime`
        :raises Exception: If the task is already stopped.
        """
        if not self.is_started:
            raise Exception("Task with ID %s is not started." % self.tid)

        self._update_duration()
        self.stopped_at = at or datetime.utcnow()
        self.is_started = False

    def _update_duration(self):
        if self.is_started:
            now = datetime.utcnow()
            delta = now - self.last_updated
            self.duration += int(delta.total_seconds())
            self.last_updated = now

    def __str__(self):
        return '<{} {}: {}>'.format(self.__class__.__name__,
                                    self.tid,
                                    self.name)

    def __eq__(self, other):
        return self.name == other.name


class TaskCollection(MutableMapping):
    """Represent a task collection dictionary-like object."""

    def __init__(self, tasks=None):
        self.store = {}
        self.update({task.tid: task for task in tasks or []})

    @classmethod
    def from_file(self, filename):
        """Create a TaskCollectiob object from a JSON file.

        :param filename: The name of the input JSON file.
        :type filename: str
        :returns: A new task collection
        :rtype: :class:`~.TaskCollection`

        :Example:

        >>> task_col = TaskCollection.from_file('example.json')
        >>> type(task_col)
        <class 'trask.models.TaskCollection'>
        """
        try:
            with open(filename, 'r') as ifile:
                task_dicts = json.load(ifile)
                task_list = [Task.from_dict(task) for task in task_dicts]
        except IOError:
            task_list = []
        return TaskCollection(tasks=task_list)

    def to_file(self, filename):
        """Export the task collection to a JSON file.

        :param filename: The name of the output JSON file.
        :type filename: str

        :Example:

        >>> task_list = [Task(name=i) for i in ['t1', 't2', 't3']])
        >>> for t in task_list:
        >>>     t.start()
        >>> time.sleep(5)
        >>> task_col = TaskCollection(tasks=task_list)
        >>> task_col.to_file('example.json')
        """
        with open(filename, 'w') as ofile:
            task_dicts = [task.to_dict() for task in self.values()]
            json.dump(task_dicts, ofile, indent=4)

    def __getitem__(self, key):
        try:
            return self.store[key]
        except KeyError:
            return self.store[generate_id(key)]

    def __setitem__(self, key, value):
        self.store[key] = value

    def __delitem__(self, key):
        del self.store[key]

    def __iter__(self):
        return iter(self.store)

    def __len__(self):
        return len(self.store)
